function [J] = EvaluateCost(theta,scenario)
% Evaluation of cost function.  Involves design of a controller based on
% the nominal model and performing a simulation of the "true" pendulum 
% model in closed-loop with this controller.
%
% Input:
%   theta           Parameter value, column vector
%   scenario        The considered scenario: imperfect model or imperfect
%                   (noisy) state measurements
%                   Choices:
%                       perturbedModel
%                       imperfectState
%
% Output:
%   J               cost, in paper notation: $\hat{J}(\theta)$
%
% --
% Self-tuning LQR, Trimpe et al. IFAC World Congress 2014 paper
%
% Copyright:
% Sebastian Trimpe
% Max-Planck-Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [13.11.13]   first version to be published


%% Program control
switch scenario
    case 'perturbedModel'
        % Scenario: perfect state measurements & imperfect model
        flag_perfectState = true;
        flag_trueModel = false;
    case 'imperfectState'
        % Scenario: perfect model, but imperfect state measurements
        flag_perfectState = false;
        flag_trueModel = true;
    otherwise
        error('Unknown user input for ''scenario''.');
end;


%% Get nominal model and cost functions definitions
% Model: true or nominal
if flag_trueModel
    % True model known for controller design
    model = ConfigPendulumModel();
else
    % True model not known, only nominal.
    model = ConfigPerturbedPendulumModel();     % nominal model
end;

% Cost definition (weighting matrices)
cost = ConfigCost();                            % weighting matrices



%% Controller design

% Try the controller design; if not feasible, warning.
try
    [F,S,~] = dlqr(model.sysd.a, model.sysd.b, cost.Q(theta), cost.R(theta));
    F = -F;     % assume positive feedback
catch err
    rethrow(err);
    warning(['LQR design failed (theta=',num2str(alpha'),'). Return inf for cost.']);
    J = inf;
    return;
end;



%% Simulation
% Call the simulation.
[x_sim, u_sim] = SimulatePendulum(F,flag_perfectState);

% If trajectory == inf (indicating unstable system), set cost to inf.
if any(any(isinf(x_sim))) || any(any(isinf(u_sim)))
    J = inf;
    return;
end;



%% Evaluate cost
K = size(x_sim,1);
J = trace(cost.Q_ref*x_sim'*x_sim) + trace(cost.R_ref*u_sim'*u_sim);
J = J/K;

% % Alternative implementation (slower)
% K = size(x_sim,1);
% J2 = 0;
% for k=1:K
%     J2 = J2 + x_sim(k,:)*cost.Q_ref*x_sim(k,:)' + u_sim(k,:)*cost.R_ref*u_sim(k,:)';
% end;
% J2 = J2/K;
