% Run_SelfTuningExperiment.m
%
% Script runs the self-tuning LQR algorithm on the illustrative
% example (simple inverted pendulum) presented in Sec. 4 of the paper:
% 
% [1] S. Trimpe, A. Millane, S. Doessegger, R. D'Andrea, "A Self-Tuning LQR
% Approach Demostrated on an Inverted Pendulum," 19th IFAC World Congress, 
% 2014.
%
% Different simulation scenarios can be chosen in section "User
% configuration" below.
%
% --
% Self-tuning LQR, Trimpe et al. IFAC World Congress 2014 paper
%
% Copyright:
% Sebastian Trimpe
% Max-Planck-Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [13.11.13]   first version to be published



%% User configuration
% Make choices about the simulation scenario here.

% Scenario: In the paper, two scenarios are considered:
%   (1) imperfect model knowledge (& perfect state measurements) 
%           -> results of Sec. 4.4
%   (2) imperfect (noisy) state measurements (& perfect model)
%           -> corresponds to Sec. 4.5 (results not shown in the paper)
% Make your choice here, which of the two shall be simulated.
scenario = 'perturbedModel';    % Options: 'perturbedModel', 'imperfectState'
%scenario = 'imperfectState';


% How many cost function evaluations?  Again, two scenarios are considered:
% (Refer to the paper for explanation of the variables.)
%   (1) small number of cost function evaluations (n_g=2, i_max=20)
%   (2) large number of cost function evaluations (n_g=10, i_max=50)
computationBudget = 'small';    % Options: 'small', 'large'
%computationBudget = 'large';


%% Perform self-tuning experiment
% Do the tuning experiment.
disp('Running self-tuning algorithm...');
[cost,theta] = SimulateSelfTuning(scenario,computationBudget,1);
disp('... done.');

% Evaluate resulting controller 100 times
disp('Analyzing result...');
[cost_avg, ~] = EvaluateCostAvg(theta,scenario,100);

% Compute resulting controller F^final
switch scenario
    case 'perturbedModel'
        model = ConfigPerturbedPendulumModel(); % perturbed pendulum model
    case 'imperfectState'
        model = ConfigPendulumModel();          % perfect pendulum model
end;
cost = ConfigCost();
F = -dlqr(model.sysd.a,model.sysd.b,cost.Q(theta),cost.R(theta));

disp('... done.');


%% Comparison: initial design
disp('Analyzing initial design...');

% Parameter
theta_init = [1; 1];

% Cost of initial design
[cost_init_avg, ~] = EvaluateCostAvg(theta_init,scenario,100);

% Controller
F_init = -dlqr(model.sysd.a,model.sysd.b,cost.Q(theta_init),cost.R(theta_init));

disp('... done.');


%% Show results
disp(' ');
disp('Results Self-Tuning Run');
disp('------------------------');
disp(' ');
disp('Initial Design');
disp(['   theta: [',num2str(theta_init(1)),' ',num2str(theta_init(2)),']']);
disp(['   gain: [',num2str(F_init(1)),' ',num2str(F_init(2)),']']);
disp(['   cost (empirical): ',num2str(cost_init_avg,'%11.4e'),'']);
disp(' ');
disp('Final Design (result of self-tuning algorithm)');
disp(['   theta: [',num2str(theta(1)),' ',num2str(theta(2)),']']);
disp(['   gain: [',num2str(F(1)),' ',num2str(F(2)),']']);
disp(['   cost (empirical): ',num2str(cost_avg,'%11.4e'),'']);